# -----------------------------------------------------------------------------

version='3.1.3'
bugs_and_fixes="""
2/26/09 - lists of points are sorted 
3/30/2011 - dependency on additional custom modules was removed.
"""

doc="""=== LineShapeKin %s ===\n
Written by Evgenii Kovrigin,    3/30/08\nMedical College of Wisconsin\n

Brief description of the interface (for complete instructions see LineShapeKin manual):
[Set current peak] - start building set of 1D slices for the selected peak.
[Assign indices] - associate (enter) numerical indices for every spectrum to use as identifier of the specific 1D slice (generally, titration point).
[Remove w2] - integrate rectangular region using w1 as X-axis and summing all points along w2.
[Remove w1] - opposite action
[Save dataset name] - write the name of the current dataset into  'matlab_peaklist.txt'. You will cut-and-paste this name into setup.m MATLAB notebook.

Bugs and fixes:
%s 
""" % (version, bugs_and_fixes)


import sputil
import tkutil
import tkMessageBox, tkFileDialog

import os
from math import *
# -----------------------------------------------------------------------------
#
#--- this is a parameter used for naming the files. Use 'es' to set it ====
INDEX_PARAMETER='relaxation_time_parameter'

dims={ 'w1':0,  # numerical order of dimensions in arrays
       'w2':1
     }


class region_rmsd_dialog(tkutil.Dialog):
  # storage for mean RMSD value
  _current_spectrum=None
  _mean_rmsd=0
  _number_of_runs=0
  def __init__(self, session):

    

    self.session = session
    self.drag_notice = None

    #--- Matlab data handling
    self.current_peak_name=''
    self.current_peak_name_saved=''
    self.matlab_path='None'

    #-- remember last spectrum where you dragged last rectangle
    self.last_spectrum=None

    #--- remember the resonance to associate all extracted datasets with
    self.current_resonance='>>> Hit  [Set current peak]'

    self.my_region=None
    
    tkutil.Dialog.__init__(self, session.tk, 'LineShapeKin %s' % version)

    self.top.columnconfigure(0, weight = 1)

    import Tkinter

    #--- 1st row label

    r = Tkinter.Label(self.top, text='Drag a box to select a peak label',justify = 'left')
    r.grid(row = 0, column = 0, sticky = 'nw')
    self.result_label = r

    #-- second row buttons
    br = tkutil.button_row(self.top,
                           ('Assign indices', self.run_es),
                           ('Set current peak', self.set_current_resonance),
    			   )
    br.frame.grid(row = 1, column = 0, sticky = 'nw')

    explain = ('Dragging a box defines the region to integrate')
    w = Tkinter.Label(self.top, text = explain, justify = 'left')
    w.grid(row = 2, column = 0, sticky = 'nw')

    #--- third row separator
    explain = ('-----------------------------------------------')
    w = Tkinter.Label(self.top, text = explain, justify = 'left')
    w.grid(row = 3, column = 0, sticky = 'nw')
    
    #--- fourth row text
    explain = ('Integrate SAME region in all spectra at once:')
    w = Tkinter.Label(self.top, text = explain, justify = 'left')
    w.grid(row = 4, column = 0, sticky = 'nw')

    #--- 5th row labels
    br = tkutil.button_row(self.top,
                           ('Remove w2', self.region_w2),
                           ('Remove w1', self.region_w1),                           
 			   )
    br.frame.grid(row = 5, column = 0, sticky = 'nw')


    #--- 6th row separator
    explain = ('-----------------------------------------------')
    w = Tkinter.Label(self.top, text = explain, justify = 'left')
    w.grid(row = 6, column = 0, sticky = 'nw')
    
    #--- 7th row text
    explain = ('Integrate region in ONE spectrum at time:')
    w = Tkinter.Label(self.top, text = explain, justify = 'left')
    w.grid(row = 7, column = 0, sticky = 'nw')

    #--- 8th row labels
    br = tkutil.button_row(self.top,
                           ('Remove w2', self.region_w2_single_spectrum),
                           ('Remove w1', self.region_w1_single_spectrum),                           
 			   )
    br.frame.grid(row = 8, column = 0, sticky = 'nw')

    #--- 9th row separator
    explain = ('-----------------------------------------------')
    w = Tkinter.Label(self.top, text = explain, justify = 'left')
    w.grid(row = 9, column = 0, sticky = 'nw')
    #-- 10th row buttons
    br = tkutil.button_row(self.top,
                            ('Save dataset name', self.append_resname),                           
                            ('Close', self.close_cb),
                            ('Help', self.help),
			   )
    br.frame.grid(row = 10, column = 0, sticky = 'nw')


    self.register_callback(1)
    self.top.bind('<Map>', lambda e, s=self: s.register_callback(1))
    self.top.bind('<Unmap>', lambda e, s=self: s.register_callback(0))



  # ---------------------------------------------------------------------------
  def help(self):
    tkMessageBox.showinfo('LineShapeKin %s'% version,message=doc)
  # ---------------------------------------------------------------------------
  def run_es(self):
    self.session.command_characters('es')
  # ---------------------------------------------------------------------------
  #
  def register_callback(self, register):

    if register:
      if self.drag_notice == None:
        self.drag_notice = self.session.notify_me('drag region',
                                                  self.region_selected_cb)
    else:
      if self.drag_notice:
        self.session.dont_notify_me(self.drag_notice)
        self.drag_notice = None

  # ---------------------------------------------------------------------------
  #
  def region_selected_cb(self, region):

    if self.is_window_destroyed():
      self.register_callback(0)
      return

    #--ek
    self.my_region=region

    s = self.session.selected_spectrum()

    #--- remember the spectrum where I defined last region
    self.last_spectrum=s
    print 'Region defined in %s' % s.name


    if s == None:
      self.result_label['text'] = 'No spectrum selected'
      return
    #print region
    rmin, rmax = region
    ranges = map(lambda vmin, vmax: '%.4g-%.4g' % (vmin, vmax), rmin, rmax)
    import string
    rtext = '[ ' + string.join(ranges, ', ') + ' ]'

    self.current_peak_name=''
    self.current_peak_name_saved=''

    self.showinfo(rtext)
        

  def showinfo(self,text):
    'to refresh info window'
    #-----------------------------------------------------------------------
    msg = ('Current peak    %s\n' % self.current_resonance +
            'Current index: %s\n' % self.last_spectrum.saved_value(INDEX_PARAMETER) +
           'Current region      %s\n' % text +
           'Saved as: %s\n%s' %(self.current_peak_name, self.current_peak_name_saved))
    self.result_label['text'] = msg

  # ---------------------------------------------------------------------------
  #

##
##  def setup_cb(self):
##
##    setup_dialog = sputil.the_dialog(relax_setup_dialog, self.session)
##    settings = {'spectrum-times': self.spectrum_times,
##                
##                'relative-scale':self.relative_scale_on_plots}
##    setup_dialog.set_parent_dialog(self, settings, self.set_params)
##    setup_dialog.show_window(1)

    
#######################################################################################
#######################################################################################
#######################################################################################
#######################################################################################

  def set_current_resonance(self):
      'choose a resonance to associate data sets with'
      selected_peaks=self.session.selected_peaks()
      if len(selected_peaks)==1:
                self.current_resonance=selected_peaks[0].assignment
      self.showinfo('')
    
      

  # ---------- integrate SAME region in ALL  spectra -----------------------------------------------------------------
  #
  def region_w1(self):
    'integrate dimension w1'
#    print self.integrate_region(self.session.selected_spectrum(),self.my_region, 'w2')
    all_spectra=self.session.project.spectrum_list()
    self.integrate_all_spectra(all_spectra, self.my_region, 'w2')
    
  def region_w2(self):
    'integrate dimension w2'
    #print self.integrate_region(self.session.selected_spectrum(),self.my_region, 'w1')
    all_spectra=self.session.project.spectrum_list()
    self.integrate_all_spectra(all_spectra, self.my_region, 'w1')

  # -------- integrate the region in ONE SPECTRUM ---------------------------------------------------------------------
  #
  def region_w2_single_spectrum(self):
    'integrate dimension w2 in ONE spectrum'    
    self.integrate_all_spectra([self.last_spectrum], self.my_region, 'w1')

  def region_w1_single_spectrum(self):
    'integrate dimension w1 in ONE spectrum'    
    self.integrate_all_spectra([self.last_spectrum], self.my_region, 'w2')




  # ------------ DO INTEGRATION OF THE REGION -----------------------------------------------------------------
  def integrate_all_spectra(self, selected_spectra, region, X_axis):

 

    path=os.path.dirname(self.session.project.save_path)
    matlab_datapath=os.path.join(path,"Data_for_MatLab") #-- goes into a current directory
    if not os.path.isdir(matlab_datapath):   os.mkdir(matlab_datapath)


    #--- make a name of folder to hold data for specific residue ---
    peaks=self.current_resonance
    peaks+="_x_"+X_axis
    peaks_path=os.path.join(matlab_datapath,peaks)
    if not os.path.isdir(peaks_path):   os.mkdir(peaks_path)


    #--- paths for MatLab data ---
    matlab_fullpath=matlab_datapath
    #--- current peak filename
    matlab_fname=peaks
    matlab_peaklist='matlab_peaklist.txt'



    
    #-- name for a file with a list of spectra
    points_fname=os.path.join(peaks_path,'points_list.txt')

    #--- Check if all spectra were integrated/exported first
    if len(selected_spectra)==1:
        if not os.path.isfile(points_fname):
              tkMessageBox.showerror("LineShapeKin","You should do integration of all spectra with the SAME region first and then do individual extraction of desired points!\n")
        
    points_file=open(points_fname,'w')
    # need to save points sorted while the spectral list is not ordered by values
    point_indices=[]
    #-- collect all indices 
    for spectrum in self.session.project.spectrum_list():
      new_index=int(spectrum.saved_value(INDEX_PARAMETER))
      if new_index in point_indices:
              tkMessageBox.showerror("LineShapeKin","Duplicate indices found: '%s' !!!\n" % new_index)
      #-- collect new index  
      point_indices.append(new_index)
      
    #-- save names of the spectra in the project in ascending order
    Npoints=len(point_indices)
    for counter in range(Npoints):
      point=min(point_indices)
      #-- save it
      points_file.write("%s\n" % point)
      #-- remove it
      point_indices.remove(point)
      
#    for spectrum in self.session.project.spectrum_list():
#      fname=spectrum.saved_value(INDEX_PARAMETER)
#      #-- collect names of the spectra in the project
#      points_file.write("%s\n" % fname)
    points_file.close()


    #-- make empty files for manual R2 and w0 (without overwriting if exists)
    temp_file=open(os.path.join(peaks_path,'manual_R2.txt'),'a')
    temp_file.close()
    temp_file=open(os.path.join(peaks_path,'manual_w0.txt'),'a')
    temp_file.close()


    #-- create file with a formal Scale Factor set to 1
    scale_factor_fname='scale_factor.txt'
    sc_path=os.path.join(peaks_path,scale_factor_fname)
    sc_file=open(sc_path,'w')
    sc_file.write('1')
    sc_file.close()
    print sc_path, '  written.'

    

    
   
    #--- run integration
    y_maxs=[]
    y_mins=[]
    datastrings=[] #-- for matlab output
    
#    for spectrum in self.session.project.spectrum_list():
    for spectrum in selected_spectra:

      print 'Integrating: %s' % spectrum.name

      print region

      
      integrated, min_X, max_X, min_Y, max_Y = self.integrate_region(spectrum,region, X_axis)
      y_maxs.append(max_Y)
      y_mins.append(min_Y)
      
      #--- plain output ----
      if spectrum.saved_value(INDEX_PARAMETER):
          fname=spectrum.saved_value(INDEX_PARAMETER)+'.txt'
      else:
          fname=spectrum.name+".no_index_assigned"
      file=open(os.path.join(peaks_path,fname),'w')


      #-- collect the spectral data
      for line in integrated:
         file.write("%f\t%f\n" % (spectrum.hz_per_ppm[dims[X_axis]]*float(line[0]), line[1]))
      file.close()
      

      #-- make sure spectrum has a point number assigned
      if spectrum.saved_value(INDEX_PARAMETER):
          index=spectrum.saved_value(INDEX_PARAMETER)
      else:
          tkMessageBox.showerror('Integrate Regions',
                                 'Need INDEX value set in each spectrum!  "%s" does not have it! '%spectrum.name)
          return
     
    

      
         


    #--- save name of exported residue to use in Matlab notebooks ---
    self.current_peak_name=matlab_fname
    self.current_peak_name_saved=''
    self.matlab_path=os.path.join(matlab_fullpath,matlab_peaklist)

    self.showinfo('')


  def append_resname(self):
    'appends the name to matlab peaklist'
    if not self.current_peak_name:
        return
    #--- save name of exported residue to use in Matlab notebooks ---
    file=open(self.matlab_path,'a')
    file.write(" '%s', " % self.current_peak_name)
    file.close()
    self.current_peak_name_saved='Appended to MatLab peaklist'
    self.showinfo('')
#---------------------------------------------------------------------------------------
  
  def integrate_region(self,spectrum, region, X_axis='w1'):
    '->integrated spectrum along "x" or "y"'

    if spectrum.dimension!=2:
        tkMessageBox.showwarning("Integrate Region","Cannot work with %d dimensions!" % spectrum.dimension)
    
    ppm_per_index = map(lambda w,s: float(w)/s,
                        spectrum.spectrum_width, spectrum.data_size)
#    print ppm_per_index

    #--- assign dimensions
    primary_dimension=dims[X_axis]
    secondary_dimension= not primary_dimension

#    print spectrum.data_height([120,8])

    
    curr_point=[0,0]

    #--- go along main dimension, integrate secondary
    
    integrated=[]

    #--- one index ---    
    min_X=region[0][primary_dimension]
    min_Y=region[0][secondary_dimension]

    max_X=region[1][primary_dimension]
    max_Y=region[1][secondary_dimension]
    
    X_steps=int((max_X-min_X)/ppm_per_index[primary_dimension])
    Y_steps=int((max_Y-min_Y)/ppm_per_index[secondary_dimension])

    # just for xmgr scaling
    max_y=spectrum.data_height(curr_point)/Y_steps 
    min_y=max_y

    for i in range(X_steps):
      
      curr_X=min_X + ppm_per_index[primary_dimension]*i
      curr_point[primary_dimension]=curr_X
      
      sum=0

      for j in range(Y_steps):

          curr_Y=min_Y + ppm_per_index[secondary_dimension]*j
          curr_point[secondary_dimension]=curr_Y
                     
          height=spectrum.data_height(curr_point)
                     
          sum+=height

      Y_point= sum/Y_steps
      integrated.append([curr_X,Y_point])
                     
      if Y_point>max_y: max_y=Y_point
      if Y_point<min_y: min_y=Y_point

    #--- ranges for plotting ---
    min_x=min_X
    max_x=min_x + ppm_per_index[primary_dimension] * (X_steps-1)
    
    
    return integrated, min_x, max_x, min_y, max_y





#-------------------------------------------------------------
  def fit_integrated(self,integrated_region, fit_func):
    'fit region to two peaks'
    pass
    
  
#######################################################################################
#######################################################################################
#######################################################################################
#######################################################################################

  
# -----------------------------------------------------------------------------
#
class lorentzian_func:
  'lorentzian lineshape peak'

  def parameter_count(self):
    return 3
  
  def value(self, params, x):
    a0, dv, v0 = params
  
    A1=a0*dv*dv
    
    A2=dv*dv-4*(x-v0)*(x-v0)

    return A1/A2


  def parameter_derivatives(self, params, x):

    epsilon=1e-6
    derivatives=[]
    
    for i in range(params):
      #-- step up ---
      temp_params2=params[:]
      temp_params2[i]=params[i]+params[i]*epsilon
      #-- step down ---      
      temp_params1=params[:]
      temp_params1[i]=params[i]-params[i]*epsilon

      
      dfda=(self.value(temp_params2,x)-self.value(temp_params1,x))/params[i]*epsilon
      
      derivatives.append(dfda)

  
    return derivatives

#  

def run(session):

  sputil.the_dialog(region_rmsd_dialog,session).show_window(1)
