# -----------------------------------------------------------------------------

version='4.0'

doc="""=== LineShapeKin %s ===\n
Written by Evgenii Kovrigin,    11/11/08\nMedical College of Wisconsin\n

Version for 2D-line shape extraction

... help text below to be updated ...


To set estimate for an intermediate state:  press F8, add a peak, press F1 and select a peak by dragging a rectangle around it (dragging operation sets the spectrum as 'current')


""" % version

import sparky
import sputil
import tkutil
import tkMessageBox, tkFileDialog
import Tkinter
import string
import glob

import os
from math import *
# -----------------------------------------------------------------------------
#
#--- this is a parameter used for naming the files. Use 'es' to set it ====
INDEX_PARAMETER='relaxation_time_parameter'  # arbitrary name used by 'es'

#-- folders and files
LineShapeKin_folder='LineShapeKin_Data'  # general place for all data

# folder and files for parameters that are constant for the titration series
constants_folder='Constants'  
B0_fname='B0.txt'
T_fname='T.txt'
nt_fname='nt.txt'
Ptotal_fname='Ptotal.txt'
LPratio_fname='LPratio.txt'

# files with data specific for every residue
data_filtering_fname='data_filtering.txt'  # set to default values
points_list_fname='points_list.txt' # list of indices in the titration
initial_state_fname='initial_state_estimate.txt'  # position and line widths in both dimensions
final_state_fname='final_state_estimate.txt'   # position and line widths in both dimensions
intermediate_state_fname='intermediate_state_estimate.txt'  # peak positions (if needed)



#================= LINESHAPEKIN DIALOG ===============

class lineshapekin_dialog(tkutil.Dialog):

  def __init__(self, session):

    

    self.session = session
    self.drag_notice = None

    #-- create folders
    path=os.path.dirname(self.session.project.save_path)
    
    #-- main folder
    self.matlab_datapath=os.path.join(path,LineShapeKin_folder) #-- goes into a current directory
    if not os.path.isdir(self.matlab_datapath):   os.mkdir(self.matlab_datapath)
    
    #-- folder for constants
    self.constants_path=os.path.join(self.matlab_datapath,constants_folder)
    if not os.path.isdir(self.constants_path): os.mkdir(self.constants_path)
    
    #-- make empty files for parameters of titration
    for fname in [B0_fname, T_fname, nt_fname, Ptotal_fname, LPratio_fname ]:
        file=os.path.join(self.constants_path,fname)
        if not os.path.isfile(file): 
            fid=open(file,'w')
            fid.close()
    
    
    
    #----------------------------------------------------

    #--- storage for multiple regions
    self.X_bounds=[]
    self.Y_bounds=[]

    #-- visual aid - lines to show what your are exporting
    self.lines=[]

    
    #-- remember last spectrum where you dragged last rectangle
    self.last_spectrum=None

    #--- remember the resonance to associate all extracted datasets with
    self.current_resonance='>>> Hit  [Set current peak]'

    self.my_region=None


    #--- bring Python Shell out for diagnostic output
    self.session.command_characters('py')

    #----  a text message to display in a dialog box as a status message 
    self.current_message=''

#===================== GENERATE A DIALOG BOX =========================================
    
    tkutil.Dialog.__init__(self, session.tk, 'LineShapeKin %s' % version)

    self.top.columnconfigure(0, weight = 1)


    #--- 1st row label

    r = Tkinter.Label(self.top, text='Drag a box to select a peak label',justify = 'left')
    r.grid(row = 0, column = 0, sticky = 'nw')
    self.result_label = r

    #-- second row buttons
    br = tkutil.button_row(self.top,
                           ('Assign indices', self.run_es),
                           ('Set current peak', self.set_current_resonance),
    			   )
    br.frame.grid(row = 1, column = 0, sticky = 'nw')

    explain = ('------ Dragging a box defines a new region -------')
    w = Tkinter.Label(self.top, text = explain, justify = 'left')
    w.grid(row = 2, column = 0, sticky = 'nw')


    #-- handling of multiple regions
    br = tkutil.button_row(self.top,
                       ('Accept new region', self.add_current_region),
 	                       ('Clear all regions', self.clear_all_regions),                           
                           ('Select all lines for deletion', self.select_all_lines),                           
                       )
    br.frame.grid(row = 3, column = 0, sticky = 'nw')


    explain = (' ')
    w = Tkinter.Label(self.top, text = explain, justify = 'left')
    w.grid(row = 4, column = 0, sticky = 'nw')
       
  
    #--- 5th row labels
    br = tkutil.button_row(self.top,
                           ('Export the SAME region from ALL spectra', self.export_same_region_all_spectra),                           
 			   )
    br.frame.grid(row = 5, column = 0, sticky = 'nw')


    
    #--- 7th row text
    br = tkutil.button_row(self.top,
                           ('Export only from the CURRENT spectrum', self.export_same_region_single_spectrum),                           
		   )
    br.frame.grid(row = 6, column = 0, sticky = 'nw')


    explain = (' ')
    w = Tkinter.Label(self.top, text = explain, justify = 'left')
    w.grid(row = 7, column = 0, sticky = 'nw')
    
    
      #------
    explain = ('----------- Estimate Positions and Line Widths of the END states ------------')
    w = Tkinter.Label(self.top, text = explain, justify = 'left')
    w.grid(row = 8, column = 0, sticky = 'nw')
    br = tkutil.button_row(self.top,
 	                       ('Save INITIAL STATE', self.set_initial_state_estimate),                           
                           ('Save FINAL STATE', self.set_final_state_estimate),                           
    )
    br.frame.grid(row = 9, column = 0, sticky = 'nw')


    #---- 
    explain = ('----------- Estimate Positions of the INTERMEDIATE states ------------')
    w = Tkinter.Label(self.top, text = explain, justify = 'left')
    w.grid(row = 10, column = 0, sticky = 'nw')
    br = tkutil.button_row(self.top,
 	                       ('Save Intermediate State', self.set_intermediate_state_estimate),                           
    )
    br.frame.grid(row = 11, column = 0, sticky = 'nw')
    br = tkutil.button_row(self.top,
                           ('Remove ALL estimates', self.remove_all_estimates),                           
    )
    br.frame.grid(row = 12, column = 0, sticky = 'nw')

    #--- 9th row separator
    explain = ('-----------------------------------------------')
    w = Tkinter.Label(self.top, text = explain, justify = 'left')
    w.grid(row = 14, column = 0, sticky = 'nw')
    #-- 10th row buttons
    br = tkutil.button_row(self.top,
   #                         ('Save dataset name', self.append_resname),                           
                            ('Close', self.close_cb),
                            ('Help', self.help),
			   )
    br.frame.grid(row = 15, column = 0, sticky = 'nw')


    self.register_callback(1)
    self.top.bind('<Map>', lambda e, s=self: s.register_callback(1))
    self.top.bind('<Unmap>', lambda e, s=self: s.register_callback(0))

#==========================================================================================================







#=========================  ADD CURRENT REGION TO A LIST OF REGIONS  (button call) =================================
  def add_current_region(self):

    #--- extract corners of the region
    minX=self.my_region[0][0] 
    maxX=self.my_region[1][0] 
    minY=self.my_region[0][1] 
    maxY=self.my_region[1][1] 

    #--- store new region
    self.X_bounds.append([minX, maxX])
    self.Y_bounds.append([minY, maxY])
     

    #-- show a rectangle box
    line=sparky.Line(self.last_spectrum, [minX,minY],[minX,maxY])
    line.color='gray'
    self.lines.append(line)
    line=sparky.Line(self.last_spectrum, [maxX,minY],[maxX,maxY])
    line.color='gray'
    self.lines.append(line)
    line=sparky.Line(self.last_spectrum, [minX,minY],[maxX,minY])
    line.color='gray'
    self.lines.append(line)
    line=sparky.Line(self.last_spectrum, [minX,maxY],[maxX,maxY])
    line.color='gray'
    self.lines.append(line)
    
    self.showinfo('')
#======================================================================================================










#============================== CLEAR ALL COLLECTED REGIONS  (button call) ============================================
  def clear_all_regions(self):
    self.Y_bounds=[]
    self.X_bounds=[]

    self.showinfo('')

#==========================================================================================================
  
  
#============================== SELECT ALL LINES IN ALL SPECTRA TO DELETE ===========================  
  def select_all_lines(self):

    #-- unselect all  first so not to accidentally delete 
    self.session.unselect_all_ornaments()

    for spectrum in self.session.project.spectrum_list():
        for line in spectrum.line_list():
            line.selected=1
#    tkMessageBox.showinfo('LineShapeKin %s'% version,message='Now hit [Delete] button to remove all lines')
        
#====================================================================================================
   
    

#===================================== HELP WINDOW  (button call) =======================================================
  def help(self):
    tkMessageBox.showinfo('LineShapeKin %s'% version,message=doc)
#==========================================================================================================
  
  
  
  
  
  
  
#======================================== INVOKE A SPECTRUM LIST TO ASSIGN INDICES ======================
  def run_es(self):
    self.session.command_characters('es')
#==========================================================================================================
  
  
  
  
  
  
    
#===================== DRAG-REGION CALLBACK  =======================================================
  def register_callback(self, register):

    if register:
      if self.drag_notice == None:
        self.drag_notice = self.session.notify_me('drag region',
                                                  self.region_selected_cb)
    else:
      if self.drag_notice:
        self.session.dont_notify_me(self.drag_notice)
        self.drag_notice = None

#==========================================================================================================






#========================== GRAB SELECTED REGION ===================================================
  def region_selected_cb(self, region):

    if self.is_window_destroyed():
      self.register_callback(0)
      return

    #-- record selected region
    self.my_region=region

    s = self.session.selected_spectrum()

    #--- remember the spectrum where I defined last region
    self.last_spectrum=s

    if s == None:
      self.result_label['text'] = 'No spectrum selected'
      return

    #--- form an output text for the newly defined region bounds
    rmin, rmax = region
    ranges = map(lambda vmin, vmax: '%.4g-%.4g' % (vmin, vmax), rmin, rmax)
    rtext = '[ ' + string.join(ranges, ', ') + ' ]'

    self.showinfo(rtext)
#==========================================================================================================        








#=========================== DISPLAY CURRENT INFO ====================================================
  def showinfo(self,text):
    'to refresh info window'
    
    #-- show defined regions
    defined_regions=''
    N=len(self.X_bounds)

    for counter in range(0,N):
        Xmin=self.X_bounds[counter][0]
        Xmax=self.X_bounds[counter][1]
        Ymin=self.Y_bounds[counter][0]
        Ymax=self.Y_bounds[counter][1]
        defined_regions+='%.4g-%.4g / %.4g-%.4g\n' % (Xmin, Xmax, Ymin, Ymax)
        
    
    #-----------------------------------------------------------------------
    msg = ('Current peak    %s\n' % self.current_resonance +
            'Current index: %s\n' % self.last_spectrum.saved_value(INDEX_PARAMETER) +
           'New region      %s\n' % text +
           '%d defined regions:\n %s' % (N,  defined_regions) +
           'Saved as: %s\n' %  self.current_resonance +
           '%s' % self.current_message)
    self.result_label['text'] = msg

#==========================================================================================================









#============================ SET NAME OF THE CURRENT DATASET  (button call) =================================================

  def set_current_resonance(self):
      'choose a resonance to associate data sets with'
      selected_peaks=self.session.selected_peaks()
      if len(selected_peaks)==1:
                self.current_resonance=selected_peaks[0].assignment
      self.showinfo('')
    
#==========================================================================================================
      
      
      
      
      
      
      
      
      
#============================ TAKE PEAK POSITION AND FITTED LINEWIDTHS AS AN INITIAL ESTIMATES =================      
  def set_initial_state_estimate(self):
    self.save_estimate(initial_state_fname)
    
    
    
    
  def set_final_state_estimate(self):
    self.save_estimate(final_state_fname)
    
    
    
    
  def set_intermediate_state_estimate(self):
    
    self.save_estimate(intermediate_state_fname, save_line_width='no', output_mode='a')



  def remove_all_estimates(self):
    estimate_files=glob.glob(os.path.join(self.matlab_datapath,self.current_resonance,'*estimate.txt'))
    for fname in  estimate_files:
        os.remove(fname)
    self.current_message='All estimates have been deleted.'
    self.showinfo('')
    
    
    
    

  #----- actual function saving the data ----

  def save_estimate(self, state_name, save_line_width='yes',output_mode='w'):
       
    if  self.last_spectrum==None:
       tkMessageBox.showerror('LineShapeKin %s'% version,message='Please, select and fit (pi) the initial state peak')
       return
         
    
    print self.last_spectrum
    peak=self.last_spectrum.selected_peaks()
    if len(peak)==0:
       tkMessageBox.showerror('LineShapeKin %s'% version,message='Please, select the fitted (pi) initial state peak')
       return
    if len(peak)>1:
       tkMessageBox.showerror('LineShapeKin %s'% version,message='Please, select and fit only ONE peak')
       return
    
    if save_line_width=='yes' and peak[0].line_width==None:
       tkMessageBox.showerror('LineShapeKin %s'% version,message='Please, perform fitting to a lorentzian shape first using "pi"!')
       return

    if save_line_width=='yes' and peak[0].volume_method!='lorentzian':
       tkMessageBox.showerror('LineShapeKin %s'% version,message='Please, select "Lorentzian" integration method in "it" dialog, hit [Apply] and fit the peak again!')
       return
    
    
    
    dirname=os.path.join(self.matlab_datapath,self.current_resonance)
    if not os.path.isdir(dirname):
       tkMessageBox.showerror('LineShapeKin %s'% version,message='Please, export spectral regions first!')
       return
        
    self.current_message='"%s" saved' % state_name
    fname='%s' % state_name
    file=open(os.path.join(dirname,fname),output_mode)
    
    
    #-- collect the spectral data
    file.write("%f\t%f\n" % (peak[0].frequency[0], peak[0].frequency[1]))
                                    
    if save_line_width=='no':
        file.close()
        self.showinfo('')
        return
        
    file.write("%f\t%f\n" % (peak[0].line_width[0], peak[0].line_width[1]))
    file.close()
    self.showinfo('')
  
#=========================================================================================


      
          
                  
      
      
#====================== EXPORT REGION FROM ALL SPECTRA (button call) ====================================
  def export_same_region_all_spectra(self):
    'integrate dimension w1'
    all_spectra=self.session.project.spectrum_list()
    self.export_all_spectra(all_spectra)
#==========================================================================================================




    
    
    
    
#====================== EXPORT REGION FROM THE CURRENT SPECTRUM  (button call)  ===========================
  def export_same_region_single_spectrum(self):
    'integrate dimension w1 in ONE spectrum'    
    self.export_all_spectra([self.last_spectrum])
#==========================================================================================================







#======================= EXPORT REGION FROM A LIST OF SPECTRA =============================================
  def export_all_spectra(self, selected_spectra):



    #--- make a name of folder to hold data for specific residue ---
    peaks=self.current_resonance
    peaks_path=os.path.join(self.matlab_datapath,peaks)
    if not os.path.isdir(peaks_path):   os.mkdir(peaks_path)

    #-- make file with the default data-filtering settings if does not exist
    fname=os.path.join(peaks_path,data_filtering_fname)
    if not os.path.isfile(fname):
        fid=open(os.path.join(peaks_path,data_filtering_fname),'w')
        fid.write('1 1')
        fid.close()
    

    #--- current peak filename
    matlab_fname=peaks

    
    
    
    #-- name for a file with a list of spectra
    points_fname=os.path.join(peaks_path,points_list_fname)

    #--- Check if all spectra were integrated/exported first
    if len(selected_spectra)==1:
        if not os.path.isfile(points_fname):
              tkMessageBox.showerror("LineShapeKin","You should do integration of all spectra with the SAME region first and then do individual extraction of desired points!\n")
        
    points_file=open(points_fname,'w')
    for spectrum in self.session.project.spectrum_list():
      fname=spectrum.saved_value(INDEX_PARAMETER)
      #-- collect indices of the spectra in the project
      points_file.write("%s\n" % fname)
    points_file.close()

    

    
   
    #--- run data extraction and export to text files  ----------
    datastrings=[] #-- for matlab output
    
    for spectrum in selected_spectra:

      print 'Exporting: %s' % spectrum.name
      

      exported = self.extract_overlapping_regions(spectrum)
      
      #-- make sure spectrum has a point number assigned
      if spectrum.saved_value(INDEX_PARAMETER):
          index=spectrum.saved_value(INDEX_PARAMETER)
      else:
          tkMessageBox.showerror('LineShapeKin',
                                 'Need INDEX value set in each spectrum!  "%s" does not have it set! '%spectrum.name)
          return

      fname=spectrum.saved_value(INDEX_PARAMETER)+'.txt'
      file=open(os.path.join(peaks_path,fname),'w')

      #-- collect the spectral data
      for line in exported:
          file.write("%f\t%f\t%f\n" % (line[0], line[1], line[2]))
      file.close()






    self.showinfo('')
#======================================================================================================














#==================== EXTRACT OVERLAPPING DISCONTINUOUS REGIONS =========================
 
  def extract_overlapping_regions(self, spectrum):
  
    if spectrum.dimension!=2:
        tkMessageBox.showwarning("LineShapeKin","Cannot work with %d dimensions!" % spectrum.dimension)
    
    ppm_per_index = map(lambda w,s: float(w)/s,
                        spectrum.spectrum_width, spectrum.data_size)

    
    curr_point=[0,0]

    
    exported=[]

    #--- determine bounds of the area encompassing all selected regions
    min_X=self.X_bounds[0][0]
    max_X=self.X_bounds[0][1]
    for line in self.X_bounds:
        if line[0]<min_X: min_X=line[0]
        if line[1]>max_X: max_X=line[1]
        
    min_Y=self.Y_bounds[0][0]
    max_Y=self.Y_bounds[0][1]
    for line in self.Y_bounds:
        if line[0]<min_Y: min_Y=line[0]
        if line[1]>max_Y: max_Y=line[1]
    
    #-- total number of regions
    N_regions=len(self.X_bounds)
        
    #--- compute number of increments to have in the spectral data ---   
        
    
    X_steps=int((max_X-min_X)/ppm_per_index[0])
    Y_steps=int((max_Y-min_Y)/ppm_per_index[1])
    steps=[X_steps, Y_steps]
    

   
    #--- do walking through the discontinous spectral regions
    #    with points connected along Y (proton, by default nomenclature)
    
    #-- this is to walk back-and-forth so to make 3D display prettier
    direction='forward'
    
    for i in range(X_steps):
      
        curr_X=min_X + ppm_per_index[0]*i
      
        for j in range(Y_steps):

          if direction=='forward':
                curr_Y=min_Y + ppm_per_index[1]*j
          else:
                curr_Y=min_Y + ppm_per_index[1]*(Y_steps-j-1)  
                              
          curr_point=[curr_X, curr_Y]
                     
          #--- decide whether the point is within any of the selected regions
          for counter in range(N_regions):
            if curr_X >= self.X_bounds[counter][0] and curr_X <= self.X_bounds[counter][1]:
                if curr_Y >= self.Y_bounds[counter][0] and curr_Y <= self.Y_bounds[counter][1]:
                    height=spectrum.data_height(curr_point)
                    #--- Export [connected_dimension, other_dimension, height]
                    exported.append([curr_Y,curr_X,height])


        #-- reverse reading direction 
        if direction=='forward': direction='reverse'
        else: direction='forward'

       
          
                
    return exported
#======================================================================================================
  





#====================== GENERIC 'RUN' FUNCTION TO BE CALLED BY SPARKY ========================

def run(session):

  sputil.the_dialog(lineshapekin_dialog,session).show_window(1)
