""" EK. new version
Because, there is no explanation how to make original version work!
"""
# -----------------------------------------------------------------------------
# Set linewidths for each of selected peaks using other peaks in spectrum with
# same assignment along the same axis.  Also move the position of the peak to
# the average position of other peaks with the same assignment for that axis.
# If there are no other peaks with the same assigment set the linewidth along
# that axis to zero and move the peak to the resonance line frequency.  If the
# peak has no assignment along an axis set the linewidth to zero for that axis.
#
# This command is used for integrating overlapped peak.  Linewidths and
# positions are copied from already integrated peaks and then you can use
# linefitting where only the peak height is adjusted with position and
# linewidth fixed.  This should produce more sensible fits than letting
# the fit adjust all linewidths and positions.
#

import sputil
import tkutil, pyutil, re
import tkMessageBox, tkFileDialog, tkSimpleDialog
import popen2,string, copy, pickle, glob

import mylib, keyfile, ek_tkutil
import os
from math import *

#--- positions of these in spectrum_times()
SPECTRUM_POS=0
INDEX_PARAMETER_POS=1
SCALE_FACTOR_POS=2
#---Just a legacy: This is the parameter in sparky files set to some value corresponding
#     to experiment you used. 
INDEX_PARAMETER='relaxation_time_parameter'
SCALE_FACTOR='scale_factor'

#-- Default fontsize for my Sparky projects --
default_font=("Helvetica",10)
larger_font=('Helvetica',11)


title="Copy Peak Positions and Linewidths"


class copy_linewidths_and_positions_dialog(tkutil.Dialog):




  def __init__(self, session):

    self.session = session

    #--- new ---

    #-- relax dialog legacy --
    self.source_spectrum_times = []
    self.destination_spectrum_times = []
    

    
    tkutil.Dialog.__init__(self, session.tk, title)

    self.top.columnconfigure(0, weight = 1)

    import Tkinter
    explain = ('Select source and destination spectra.')
    w = Tkinter.Label(self.top, text = explain, justify = 'left', font=default_font)
    w.grid(row = 0, column = 0, sticky = 'nw')



    #--- region line ---
    br = ek_tkutil.button_row(self.top,
                           ('Select source spectrum', self.select_source),
                           ('Select destination spectra', self.select_target),
                           ('Copy linewidths and positions', self.copy),
                           ('Close', self.close_cb),
                           ('Help', self.help),
			   )
    br.frame.grid(row = 1, column = 0, sticky = 'nw')

    #-- main info ---
    r = Tkinter.Label(self.top, justify = 'left', font=larger_font)
    r.grid(row = 3, column = 0, sticky = 'nw')
    self.result_label = r



  #---- menu: Selecting source spectra
  #
  def select_source(self):
    #-- a resulting spectrum list gets assigned to self.spectrum_times[]

    setup_dialog = sputil.the_dialog(relax_setup_dialog, self.session)
    settings = {'spectrum-times': self.source_spectrum_times}
    setup_dialog.set_parent_dialog(self, settings, self.set_source_params)
    setup_dialog.show_window(1)


  def set_source_params(self, settings):
    'actually assign a new spectrum list'
    self.source_spectrum_times = settings['spectrum-times'] 
    if len(self.source_spectrum_times)>1:
      tkMessageBox.showerror(title,"You should select only ONE source spectrum!")
      return

    self.refresh_info()   

  #---- menu: Selecting target spectra
  #
  def select_target(self):
    #-- a resulting spectrum list gets assigned to self.spectrum_times[]

    setup_dialog = sputil.the_dialog(relax_setup_dialog, self.session)
    settings = {'spectrum-times': self.destination_spectrum_times}
    setup_dialog.set_parent_dialog(self, settings, self.set_target_params)
    setup_dialog.show_window(1)


  def set_target_params(self, settings):
    'actually assign a new spectrum list'
    self.destination_spectrum_times = settings['spectrum-times'] 
    self.refresh_info()   

  #----- REFRESH INFO --------
    
  def refresh_info(self,additional_text=""):
    'Refresh message in the main window'

    
    msg=""
    
    
    if self.source_spectrum_times:

        
      msg+="Source spectrum:\n%s\n" % self.source_spectrum_times[0][SPECTRUM_POS].name

    if self.destination_spectrum_times:
      msg+="Destination spectra:\n"
      for spectrum in self.destination_spectrum_times:
        msg+=spectrum[SPECTRUM_POS].name+'\n'

    self.result_label['text'] = msg + additional_text +'\n'
    
    
  #-------- menu:  My help function ----------
  def help(self):
    tkMessageBox.showinfo(title,
    """Written by Evgenii Kovrigin 05/19/05
    Allows one to copy linewidths and positions from one selected spectrum to other ones.
    """ )
    


  # ------------ COPY -----------------------------------------------------------------
  #
  def copy(self):

    source = self.source_spectrum_times[0][SPECTRUM_POS]
    print 'Copying peak positions and linewidths from: %s' % source.name
    
    for spectrum in self.destination_spectrum_times:
      destination=spectrum[SPECTRUM_POS]
      print 'To: %s ' % destination.name
      
      peaks = source.selected_peaks()
      if not peaks:
        tkMessageBox.showerror(title,"No peaks selected in the source spectrum!")
        return
        
      for peak in peaks:
        print peak.assignment,
        
        f=peak.frequency
        lw=peak.line_width
        
        destination_peak=destination.find_peak(peak.assignment)

        if not destination_peak:
          tkMessageBox.showerror(title,"Peak '%s' not found in the destination spectrum '%s'! Copy assignments first!" %(
            peak.assignment,destination.name))
          return
          
        destination_peak.frequency = f
        destination_peak.line_width = lw
        destination_peak.line_width_method = 'copy'
        
      print

    self.refresh_info("%d peaks, %d destination spectra.\nDone." %(
                      len(peaks),
                      len(self.destination_spectrum_times)))
                      

    

#






#---------------------------------------------------------------------------------------
class relax_setup_dialog(tkutil.Settings_Dialog):

  def __init__(self, session):

    tkutil.Settings_Dialog.__init__(self, session.tk, 'Project Spectra')

    headings = ('Spectrum  ', 'Time parameter', 'Scale factor')
    st = sputil.spectrum_table(session, self.top, headings,
                               self.add_spectrum, self.remove_spectrum)
    st.frame.pack(side = 'top', anchor = 'w')
    self.spectrum_widgets = {}

    spectra = session.project.spectrum_list()
    def time_param(s, self=self):
      return pyutil.string_to_float(self.default_time_parameter(s), 0)
    spectra = pyutil.sort_by_function_value(spectra, time_param)
    for spectrum in spectra:
      st.add_spectrum(spectrum)

   
    br = ek_tkutil.button_row(self.top,
                           ('Ok', self.ok_cb),
                           ('Apply', self.apply_cb),                           
                           ('Close', self.close_cb),
                           #('Help', sputil.help_cb(session, 'RelaxFit')),
			   )
    br.frame.pack(side = 'top', anchor = 'w')

  # ---------------------------------------------------------------------------
  #
  def add_spectrum(self, spectrum, table, row):

    onoff = self.default_onoff(spectrum)

    b = tkutil.checkbutton(table.frame, spectrum.name, onoff)
    b.button.grid(row = row, column = 0, sticky = 'nw')

    ttext = self.default_time_parameter(spectrum)
    e = tkutil.entry_field(table.frame, '', ttext, trailer='s')
    e.frame.grid(row = row, column = 1, sticky = 'nw')

    ftext=spectrum.saved_value(SCALE_FACTOR)
    if ftext==None: ftext=1
    f = tkutil.entry_field(table.frame, 'Intensity *', ftext, width=4)
    f.frame.grid(row = row, column = 2, sticky = 'nw')

    self.spectrum_widgets[spectrum] = (b, e, f)

  # ---------------------------------------------------------------------------
  #
  def remove_spectrum(self, spectrum, table):

    b, e = self.spectrum_widgets[spectrum]
    del self.spectrum_widgets[spectrum]
    b.button.destroy()
    e.frame.destroy()
    
  # ---------------------------------------------------------------------------
  #
  def default_onoff(self, spectrum):

    return spectrum.saved_value(INDEX_PARAMETER) != None
  
  # ---------------------------------------------------------------------------
  #
  def default_time_parameter(self, spectrum):

    #------ get time parameter --- 
    ttext = spectrum.saved_value(INDEX_PARAMETER)
    if ttext:
      return ttext
    
    #--- automatic time determination --
    match = re.search('[0-9]+', spectrum.name)
    if match:
      return match.group(0)
    
    return '0'

  # ---------------------------------------------------------------------------
  #
  def show_settings(self, settings):

    stimes = {}
    for s, t, f in settings['spectrum-times']:
      stimes[s] = (t, f)
      
    for s, (b, te, fe) in self.spectrum_widgets.items():
      b.variable.set(stimes.has_key(s))
      if stimes.has_key(s) :
	t, f = stimes[s]
        if pyutil.string_to_float(te.variable.get()) != t:
          te.variable.set('%.5g' % t)
        if pyutil.string_to_float(fe.variable.get()) != f:
          fe.variable.set('%.5g' % f)
      
  # ---------------------------------------------------------------------------
  #
  def get_settings(self):

    settings = {'spectrum-times': self.spectrum_times() }
    return settings
    
  # ---------------------------------------------------------------------------
  #
  def spectrum_times(self):

    stf = []
    for s, (b, te, fe) in self.spectrum_widgets.items():
      if b.variable.get():
        ttext = te.variable.get()
        t = pyutil.string_to_float(ttext)
        ftext = fe.variable.get()
        f = pyutil.string_to_float(ftext, 1.0)


        stf.append((s, t, f))
        
        s.save_value(INDEX_PARAMETER, ttext)
        s.save_value(SCALE_FACTOR,str(ftext))
    stf.sort(self.spectrum_time_order)
    return tuple(stf)

  # ---------------------------------------------------------------------------

  # ---------------------------------------------------------------------------
  #
  def spectrum_time_order(self, st1, st2):

    return cmp(st1[1], st2[1]) or cmp(st1[0].name, st2[0].name)







def run(session):

  sputil.the_dialog( copy_linewidths_and_positions_dialog,session).show_window(1)


















