doc="""Display_interleaved_points
=================
This extension is based on Tom's 'relax.py'.
modified by Evgenii Kovrigin, 02/25/05 to allow simple analysis
of series of spectra.

Specific features:

- uses 'Index' to allow entry of either relaxation times or interleave indices

- Displays noise value entered in the setup dialog on the plot as red and blue bars (+value, -value) give you an idea of significance of the points scatter

- Saves data in simple format with heading containing index values

- Data heights measured by 3x3 grid so - dependen on Zero-Filling!

Written for processing data obtained with 'ek_13C_rc_cpmg_intl.c'
"""
#-------------------------------------------------------------------------

developer_info="""========  INFORMATION FOR A DEVELOPER ==================
This code contains many unused functions inherited from relax_dialog and
data fitting routines. One can get rid of them.
"""



import curvefit3
import string
import copy
import math, os
import re
import string
import Tkinter
import tkFileDialog
import tkMessageBox
import curvefit3 as curvefit
import curvefit3_functions
import pyutil
import sparky
import sputil
import tkutil

import residues

# -----------------------------------------------------------------------------
#
class relaxation_dialog(tkutil.Dialog, tkutil.Stoppable):

  def __init__(self, session, title='Fitting of peak intensities decay'):

    self.session = session
    self.spectrum_times = []
    self.last_spectrum_times = []
    self.error_estimate_trials = 50
    self.relative_scale_on_plots= session.selected_spectrum().noise*3

    #-- for saving in a brief format
    self.brief_peak_list=residues.Annn_Num()

    #-- for looking through the fits using arrow keys ---
    self.current_fit_index=-1 # no fit chosen 
    self.my_fit_list=[]
    
    tkutil.Dialog.__init__(self, session.tk, title)

    pl = tkutil.scrolling_list(self.top, heading = '', height = 5)
    pl.frame.pack(side = 'top', fill = 'both', expand = 1)
    pl.listbox.bind('<ButtonRelease-1>', self.show_fit_cb)
    pl.listbox.bind('<KeyPress-Delete>', pl.delete_selected_cb)
    # The following is needed so key press is received by list box.
    pl.listbox.bind('<ButtonPress-1>', pl.set_focus_cb)
    self.peak_list = pl

    pp = tkutil.checkbutton(self.top,
        'Measure heights in all spectra at the ppm position of peaks in the active window', 0)
    self.single_peak_position = pp.variable
    pp.button.pack(side = 'top', anchor = 'w')

    progress_label = Tkinter.Label(self.top, anchor = 'nw')
    progress_label.pack(side = 'top', anchor = 'w')

    br = tkutil.button_row(self.top,
                           ('Setup...', self.setup_cb),
                           ('Save', self.peak_list.save_cb),        # res-R-err heights file
                           ('Append', self.peak_list.append_cb),
                           ('Stop', self.stop_cb), 
                           ('Clear', self.clear_cb),
                           
			   ('Close', self.close_cb),
                           ('Help', self.my_help),
                           ('Show next plot', self.show_next_fit),
			   ('Show prev plot', self.show_previous_fit),
                           )
    br.frame.pack(side = 'top', anchor = 'w')

    tkutil.Stoppable.__init__(self, progress_label, br.buttons[3])

  # ---------------------------------------------------------------------------
  #
  def my_help(self):
    tkMessageBox.showinfo("Help",message=doc)
#    self.session.command_characters('py')
    print doc
  # ---------------------------------------------------------------------------
  #
  
  def show_next_fit(self):

    if self.current_fit_index==-1: return # not initialized yet
    
    if not self.current_fit_index<(len(self.my_fit_list)-1): return # end reached

    self.current_fit_index+=1
    fit=self.my_fit_list[self.current_fit_index]
    curvefit.show_fit(self.session, fit.name, fit, self.relative_scale_on_plots,  self.show_peak_cb)
  
  # ---------------------------------------------------------------------------
  #
  def show_previous_fit(self):

    if self.current_fit_index==-1: return  # no fit chosen yet

    
    if not self.current_fit_index>0: return # start reached

    self.current_fit_index-=1
    fit=self.my_fit_list[self.current_fit_index]
    curvefit.show_fit(self.session, fit.name, fit, self.relative_scale_on_plots,  self.show_peak_cb)
      
    


  # ---------------------------------------------------------------------------
  #


  def setup_cb(self):

    setup_dialog = sputil.the_dialog(relax_setup_dialog, self.session)
    settings = {'spectrum-times': self.spectrum_times,
                
                'relative-scale':self.relative_scale_on_plots}
    setup_dialog.set_parent_dialog(self, settings, self.set_params)
    setup_dialog.show_window(1)

  # ---------------------------------------------------------------------------
  #
  def set_params(self, settings):

    self.spectrum_times = settings['spectrum-times']
    self.relative_scale_on_plots=settings['relative-scale']


  # ---------------------------------------------------------------------------
  #
  def clear_cb(self):

    self.peak_list.clear() # clear long list
    self.brief_peak_list=residues.Annn_Num() # short list

  # ---------------------------------------------------------------------------
  #
  def show_fit_cb(self, event):

    fit = self.peak_list.event_line_data(event)
    if fit:
      curvefit.show_fit(self.session, fit.name, fit, self.relative_scale_on_plots, self.show_peak_cb)
      self.current_fit_index=self.my_fit_list.index(fit)

  # ---------------------------------------------------------------------------
  #
  def show_peak_cb(self, fit, n):

    sputil.show_spectrum_position(fit.spectrum_times[n][0], fit.peak_position)
    
  # ---------------------------------------------------------------------------
  #
  def show_peak_heights(self):
    if not self.top.winfo_ismapped():
      self.show_window(1)
    if self.spectrum_times:
      if self.spectrum_times != self.last_spectrum_times:
        self.last_spectrum_times = self.spectrum_times
        self.show_heading(self.spectrum_times)
      self.stoppable_call(self.show_peaks,
                          self.session.selected_peaks(),
                          self.spectrum_times)
  # ---------------------------------------------------------------------------
  #
  def show_peaks(self, peaks, spectrum_times):

    single_position = self.single_peak_position.get()
    self.stoppable_loop(' peaks', 1)
    for p in self.session.selected_peaks():
      self.check_for_stop()
      if single_position or not p.is_assigned:
	assignment = None
      else:
	assignment = p.assignment
      th_pairs = self.time_height_pairs(p.position, assignment, spectrum_times)
      fit = self.fit_function(th_pairs, assignment)
      fit.name = p.assignment
      fit.peak_position = p.position
      fit.spectrum_times = spectrum_times
      line = self.peak_line(p, fit)
      self.peak_list.append(line, fit)
      self.peak_list.listbox.see('end')
      #-- append short list for saving and for viewing
      self.my_fit_list.append(fit)
      self.brief_peak_list.add(p.assignment[:-3],[-fit.params[1], fit.param_sd[1]])
  # ---------------------------------------------------------------------------
  #
  #
  def time_height_pairs(self, pos, assignment, spectrum_times):

    th_pairs = []

    for s, t in spectrum_times:
      if assignment:
	# Use position of peak with matching assignment if available
	peak = s.find_peak(assignment)
      else:
	peak = None
      if peak:
	h = self.average_height(s, peak.position)
      else:
	h = self.average_height(s, pos)
      th_pairs.append([t, h])
    return th_pairs
# -------------------------------------------------------------------------  
#
  def average_height(self, spectrum, pos):
   
    xstep = spectrum.spectrum_width[0] / spectrum.data_size[0]
    ystep = spectrum.spectrum_width[1] / spectrum.data_size[1]

    #-- 3x3 grid averaging
##    sum = 0
##    for i in (-1, 0, 1):
##      for j in (-1, 0, 1):
##        p = (pos[0] +i * xstep, pos[1] + j * ystep)
##	sum = sum + spectrum.data_height(p)
##    h = sum / 9.0

    #-- alternative measure: height at one point  
    h = spectrum.data_height(pos)

    return h 

  # ---------------------------------------------------------------------------
  #
  def fit_function(self, th_pairs, assignment):
    
    

    h_values = map(lambda th: th[1], th_pairs)
    h_middle = .5 * (max(h_values) + min(h_values))

    
    params = (h_middle,0)
    trials = self.error_estimate_trials
    epsilon=1e-10

    #-- run my Any_Function object now -----
    func = curvefit3_functions.Exponent_Func2()
    func.set_data(th_pairs)
    func.set_init_params(params)

    #-- try fitting ---   
    try:
      func.fit_data(trials,epsilon)
    except OverflowError:
      self.session.command_characters('py')
      print "\n\nFitting warning!   Assignment %s didn't fit !!!" % assignment
      func.fit.param_sd=[0]*len(func.fit.params)
      return func.fit

    
    #-- try error estimate ---
    try:
      func.set_same_dy(func.fit.point_sd)
      func.fit.estimate_parameter_error(trials)
    except OverflowError:
      self.session.command_characters('py')
      print "\n\nFitting warning!   Error estimate for %s failed !!!" % assignment
      func.fit.param_sd=[0]*len(func.fit.params)

    return func.fit

  # ---------------------------------------------------------------------------
  # The error estimate for T is not the standard deviation over fits with
  # gaussian random y errors added.  I believe such a standard deviation is
  # divergent because the flat fit (T = infinity) case has codimension 1.
  # So instead I take the SD of the rate constant (1/T) and convert this
  # to an equivalent T error.
  #
  def calculate_decay_time(self, fit):

    if fit.converged and fit.params[1] != 0:
      t_decay = -1 / fit.params[1]
      t_sd = abs(t_decay * fit.param_sd[1] /  fit.params[1])
    else:
      t_decay = 0
      t_sd = 0
    
    fit.t_decay = t_decay
    fit.t_sd = t_sd

  # ---------------------------------------------------------------------------
  #
 
class relax_setup_dialog(tkutil.Settings_Dialog):

  def __init__(self, session):

    tkutil.Settings_Dialog.__init__(self, session.tk, 'Spectra series v.1')

    headings = ('Spectrum  ', 'Index')
    st = sputil.spectrum_table(session, self.top, headings,
                               self.add_spectrum, self.remove_spectrum)
    st.frame.pack(side = 'top', anchor = 'w')
    self.spectrum_widgets = {}

    spectra = session.project.spectrum_list()
    def time_param(s, self=self):
      return pyutil.string_to_float(self.default_time_parameter(s), 0)
    spectra = pyutil.sort_by_function_value(spectra, time_param)
    for spectrum in spectra:
      st.add_spectrum(spectrum)



    r = tkutil.entry_field(self.top, 'Relative scale on plots:', '25000', 10, "(3xRMSD is noise peak level)")
    r.frame.pack(side = 'top', anchor = 'w')
    self.relative_scale = r



   
    br = tkutil.button_row(self.top,
                           ('Ok', self.ok_cb),
                           ('Apply', self.apply_cb),                           
                           ('Close', self.close_cb),
			   )
    br.frame.pack(side = 'top', anchor = 'w')

  # ---------------------------------------------------------------------------
  #
  def add_spectrum(self, spectrum, table, row):

    onoff = self.default_onoff(spectrum)

    b = tkutil.checkbutton(table.frame, spectrum.name, onoff)
    b.button.grid(row = row, column = 0, sticky = 'nw')

    ttext = self.default_time_parameter(spectrum)
    e = tkutil.entry_field(table.frame, '', ttext, trailer='')
    e.frame.grid(row = row, column = 1, sticky = 'nw')

    ftext=1

    self.spectrum_widgets[spectrum] = (b, e)

  # ---------------------------------------------------------------------------
  #
  def remove_spectrum(self, spectrum, table):

    b, e = self.spectrum_widgets[spectrum]
    del self.spectrum_widgets[spectrum]
    b.button.destroy()
    e.frame.destroy()
    
  # ---------------------------------------------------------------------------
  #
  def default_onoff(self, spectrum):

    return spectrum.saved_value('relaxation_time_parameter') != None
  
  # ---------------------------------------------------------------------------
  #
  def default_time_parameter(self, spectrum):

    ttext = spectrum.saved_value('relaxation_time_parameter')
    if ttext:
      return ttext
    
    match = re.search('[0-9]+', spectrum.name)
    if match:
      return match.group(0)
    
    return '0'

  # ---------------------------------------------------------------------------
  #
  def show_settings(self, settings):

    stimes = {}
    for s, t in settings['spectrum-times']:
      stimes[s] = (t)
      
    for s, (b, te) in self.spectrum_widgets.items():
      b.variable.set(stimes.has_key(s))
      if stimes.has_key(s) :
	t= stimes[s]
        if pyutil.string_to_float(te.variable.get()) != t:
          te.variable.set('%.5g' % t)
      

    self.relative_scale.variable.set('%d' % settings['relative-scale'])
  # ---------------------------------------------------------------------------
  #
  def get_settings(self):


    
    relative_scale=pyutil.string_to_int(self.relative_scale.variable.get(), 0)
    
    settings = {'spectrum-times': self.spectrum_times(),

                'relative-scale':relative_scale}
    return settings
    
  # ---------------------------------------------------------------------------
  #
  def spectrum_times(self):

    stf = []
    for s, (b, te) in self.spectrum_widgets.items():
      if b.variable.get():
        ttext = te.variable.get()
        t = pyutil.string_to_float(ttext)
        stf.append((s, t))
        s.save_value('relaxation_time_parameter', ttext)
    stf.sort(self.spectrum_time_order)
    return tuple(stf)

  # ---------------------------------------------------------------------------
  #
  def spectrum_time_order(self, st1, st2):

    return cmp(st1[1], st2[1]) or cmp(st1[0].name, st2[0].name)

# -----------------------------------------------------------------------------
#

#=================================================================
class no_real_fit(curvefit3.least_squares_fit):
  'just to create the fit object without fitting'
   
  # ---------------------------------------------------------------------------
  def find_fit(self):
     return 1
  # ---------------------------------------------------------------------------
  def calculate_error(self, params):
     return 1
  # ---------------------------------------------------------------------------
  def estimate_parameter_error(self, trials):
     pass
#=================================================================

class data_display(curvefit3.fit_display):
  'just to show the array of data'
  # ---------------------------------------------------------------------------
  #
  def plot_fit(self, heading, fit,  relative_scale, callback = None):

    mark_height= relative_scale # simple measure of height
    
    self.heading['text'] = heading

    x_range, y_range = self.xy_ranges(fit.xy_pairs)

    c = self.canvas
    c.delete('all')
    c.update_idletasks()   # Make sure geometry has been calculated
    w = c.winfo_width()
    h = c.winfo_height()
    r = 5
    
  
    for k in range(len(fit.xy_pairs)):
      x, y, dy = fit.xy_pairs[k]

      px = pyutil.map_interval(x, x_range, (0,w))
      py = pyutil.map_interval(y, y_range, (h,0))
      id = c.create_oval(px-r, py-r, px+r, py+r, fill = 'black')
      

     
      if callback:
        cb = lambda event, fit=fit, k=k, callback=callback: callback(fit, k)
        c.tag_bind(id, '<ButtonPress>', cb)

    yzero=pyutil.map_interval(0, y_range, (h,0))

    #-- prevent the marker going out of range
    if mark_height>y_range[1]:
      ymarkplus=0
    else:
      ymarkplus=pyutil.map_interval(mark_height, y_range, (h,0))

    if mark_height<y_range[0]:
      ymarkminus=h
    else:
      ymarkminus=pyutil.map_interval(-mark_height, y_range, (h,0))

    markplus=c.create_rectangle(0, yzero, 0+2*r, ymarkplus, fill = 'red')
    markminus=c.create_rectangle(0, ymarkminus, 0+2*r, yzero, fill = 'blue')

    c.update_idletasks()
    
  # ---------------------------------------------------------------------------
  #


#=================================================================

def show_fit(session, heading, fit, relative_scale, callback = None):
  d = sputil.the_dialog(data_display, session)
  d.show_window(1)
  d.plot_fit(heading, fit, relative_scale, callback)

#=================================================================

class dummy_function(curvefit3_functions.Any_Func):
  'this is a fake function to allow just data display'
  def fit_data(self,error_estimate_trials=10, epsilon=1e-6):

      self.fit = no_real_fit(self.data, self, self.init_params, epsilon)
  def parameter_count(self):
    return 1
  
  def value(self, params, x):
    return params

#=================================================================


class display_peak_heights_dialog( relaxation_dialog):
   def __init__(self, session,
                title='Displaying the peak heights vs. interleave index'):
      relaxation_dialog.__init__(self, session, title)
     
 # ---------------------------------------------------------------------------
  #
   def fit_function(self, th_pairs, assignment):

    h_values = map(lambda th: th[1], th_pairs)
    h_middle = .25 * (max(h_values) + min(h_values))

    
    params = (h_middle,-1,5*h_middle,-50)
    trials = self.error_estimate_trials
    epsilon=1e-10

    #-- run my Any_Function object now -----
    func = dummy_function()
    func.set_data(th_pairs)
    func.set_init_params(params)

    #-- try fitting ---   
    func.fit_data(trials,epsilon)
    return func.fit
  # ---------------------------------------------------------------------------
  #
   def show_fit_cb(self, event):

    fit = self.peak_list.event_line_data(event)
    if fit:
      show_fit(self.session, fit.name, fit, self.relative_scale_on_plots, self.show_peak_cb)
      self.current_fit_index=self.my_fit_list.index(fit)
  # ---------------------------------------------------------------------------
  #
   def show_next_fit(self):

    if self.current_fit_index==-1: return # not initialized yet
    
    if not self.current_fit_index<(len(self.my_fit_list)-1): return # end reached

    self.current_fit_index+=1
    fit=self.my_fit_list[self.current_fit_index]
    show_fit(self.session, fit.name, fit, self.relative_scale_on_plots,  self.show_peak_cb)
  
  # ---------------------------------------------------------------------------
  #
   def show_previous_fit(self):

    if self.current_fit_index==-1: return  # no fit chosen yet

    
    if not self.current_fit_index>0: return # start reached

    self.current_fit_index-=1
    fit=self.my_fit_list[self.current_fit_index]
    show_fit(self.session, fit.name, fit, self.relative_scale_on_plots,  self.show_peak_cb)

 
 

#==========================================================================================
   def peak_line(self, peak, fit):

     htext = ''
     for t, h, dh in fit.xy_pairs:
       htext = htext + '%-13.6g ' % h
 
     line = ('%-13s' % (peak.assignment)  + htext)
     return line
#==========================================================================================  
   def show_heading(self, spectrum_times):

     self.peak_list.clear()
     heading = '%-13s' % ('Index:')
     for s, t in spectrum_times:
       st = '%g' %  t 
       heading = heading + '%-13s ' % st
     self.peak_list.heading['text'] = heading
#==========================================================================================
   def show_peaks(self, peaks, spectrum_times):

     single_position = self.single_peak_position.get()
     self.stoppable_loop(' peaks', 1)
     for p in self.session.selected_peaks():
       self.check_for_stop()
       if single_position or not p.is_assigned:
         assignment = None
       else:
	 assignment = p.assignment
       th_pairs = self.time_height_pairs(p.position, assignment, spectrum_times)
       fit = self.fit_function(th_pairs, assignment)
       fit.name = p.assignment
       fit.peak_position = p.position
       fit.spectrum_times = spectrum_times
       line = self.peak_line(p, fit)
       self.peak_list.append(line, fit)
       self.peak_list.listbox.see('end')
       #-- append short list for saving and for viewing
       self.my_fit_list.append(fit)
      
       self.brief_peak_list.add(p.assignment[:-3],list(map(lambda xy: xy[1], fit.xy_pairs)))

  # ---------------------------------------------------------------------------
#==========================================================================================


def run(session):
  sputil.the_dialog(display_peak_heights_dialog,session).show_peak_heights()
