# -----------------------------------------------------------------------------

version='1.0'

doc="""


=== Export peak snapshot %s ===
Written by Evgenii Kovrigin,    2/19/09
Medical College of Wisconsin

 Extraction of rectangular region around every selected peak in the spectrum
 
 It makes a folder called by a spectrum name .peaks.
 The following files are saved for each selected peak in a current spectrum:
    peak_name.dat - rectangular matrix of heights
    peak_name.x - x-vector of coordinates in the matrix
    peak_name.y - y-vector of coordinates
    peak_name.neighbors - a list of peaks with frequencies found in the defined region

 Next you can run 'dat2png' Matlab program to convert all or specific .dat 
 files into small PNG files suitable for embedding in HTML documents.

""" % version

import sparky
import sputil
import tkutil
import tkMessageBox, tkFileDialog
import Tkinter
import string
import glob

import os
from math import *



def run(session):
    """ Extract 2D line shapes for all peaks"""

    session.command_characters('py')
    print doc
    
    
    #-- take current spectrum
    peaks=session.selected_peaks()
    spectrum=session.selected_spectrum()
    spectrum_name=os.path.basename(spectrum.save_path)
    path=os.path.join(session.project.sparky_directory,spectrum_name+'.peaks')
    if not os.path.isdir(path): os.mkdir(path)


	
    #-- define an area to extrac
    dw1ppm=1
    dw2ppm=0.1
    
    #-- extract area around the peak
    ppm_per_index = map(lambda w,s: float(w)/s,
                        spectrum.spectrum_width, spectrum.data_size)
    for peak in peaks:
        #-- write region to a disk
        extract_region(path,spectrum,peak,[dw1ppm,dw2ppm])

      

    
    print "%d peaks extracted." % len(peaks)

   

   
#==================== EXTRACT RECTANGULAR REGION =========================
 
def extract_region(path,spectrum,peak,dw):
    """ This procedure performs extraction of spectral data from the raw data matrix.
    The key problem is that all data is only defined for specific grid 
    points but we don't have means to directly access grid coordinates. 
    Therefore we need to recalculate grid coordinates and for the desired 
    region around the peak select closest grid points.
    """
    
    # Extract proton dimension as X, nitrogen as Y"
    Xdim=1
    Ydim=0
    
    
    #-- determine frequency difference between the grid points
    dw_grid = map(lambda w,s: float(w)/s,
                        spectrum.spectrum_width, spectrum.data_size)

    #-- find index corresponding to the lower boundary of the region around the peak
    ind_min=[0,0]
    ind_min[Xdim]=(peak.position[Xdim] - dw[Xdim] - spectrum.region[0][Xdim])/dw_grid[Xdim] +1
    ind_min[Ydim]=(peak.position[Ydim] - dw[Ydim] - spectrum.region[0][Ydim])/dw_grid[Ydim] +1
    # make sure lower boundary does not go outside of the spectrum
    if ind_min[Xdim]<1: ind_min[Xdim]=1
    if ind_min[Ydim]<1: ind_min[Xdim]=1
    

    #--- calculate closest grid point (integer)
    i_grid_min=[0,0]
    i_grid_min[Xdim]=int(round(ind_min[Xdim]))
    i_grid_min[Ydim]=int(round(ind_min[Ydim]))
    
    #---- calculate frequency of the grid point closest to the lower boundary
    omega_min=[0,0]
    omega_min[Xdim]= spectrum.region[0][Xdim] + dw_grid[Xdim]*(i_grid_min[Xdim] - 1)
    omega_min[Ydim]= spectrum.region[0][Ydim] + dw_grid[Ydim]*(i_grid_min[Ydim] - 1)


    #---- calculate index corresponding the upper boundary of the reqion around the peak
    ind_max=[0,0]
    ind_max[Xdim]=(peak.position[Xdim] + dw[Xdim] - spectrum.region[0][Xdim])/dw_grid[Xdim] +1
    ind_max[Ydim]=(peak.position[Ydim] + dw[Ydim] - spectrum.region[0][Ydim])/dw_grid[Ydim] +1
    # make sure lower boundary does not go outside of the spectrum
    if ind_max[Xdim]>spectrum.data_size[Xdim]: ind_max[Xdim]=spectrum.data_size[Xdim]
    if ind_max[Ydim]>spectrum.data_size[Ydim]: ind_max[Xdim]=spectrum.data_size[Ydim]
     
    #--- calculate closest grid point (integer)
    i_grid_max=[0,0]
    i_grid_max[Xdim]=int(round(ind_max[Xdim]))
    i_grid_max[Ydim]=int(round(ind_max[Ydim]))

    
    #---- calculate frequency of the grid point closest to the lower boundary
    omega_max=[0,0]
    omega_max[Xdim]= spectrum.region[0][Xdim] + dw_grid[Xdim]*(i_grid_max[Xdim] - 1)
    omega_max[Ydim]= spectrum.region[0][Ydim] + dw_grid[Ydim]*(i_grid_max[Ydim] - 1)


    #--- generate a list of indices for the region 
    di=[0,0]
    di[Xdim]=i_grid_max[Xdim]-i_grid_min[Xdim]
    di[Ydim]=i_grid_max[Ydim]-i_grid_min[Ydim]


    
    di_list=[[],[]]
    di_list[Xdim]=range(0,di[Xdim],1)
    di_list[Ydim]=range(0,di[Ydim],1)

    
    #---- generate a list of frequencies and prepare their output
    dw_list=[[],[]]
    X_line="" # row vector of X values for the matrix
    for index in di_list[Xdim]:
       curr_X=index*dw_grid[Xdim]+ omega_min[Xdim]
       dw_list[Xdim].append( curr_X )
       X_line+="%f\n" % curr_X
       
    Y_line="" # row vector of Y values for the matrix
    for index in di_list[Ydim]:
       curr_Y=index*dw_grid[Ydim]+ omega_min[Ydim]
       dw_list[Ydim].append( curr_Y )
       Y_line+="%f\n" % curr_Y

    #--- extract spectral data from the calculated grid points
    output="" # rectangular matrix of all heights  
    curr_point=[0,0]
    for dwX in dw_list[Xdim]:
      for dwY in dw_list[Ydim]:
          
          curr_point[Xdim]=dwX           
          curr_point[Ydim]=dwY
                     
          height=spectrum.data_height(curr_point)
          
          output+="%f\t" % (height)
                    
      output+="\n" # close the row


    #--- export for plotting ---
    #- rectangular matrix
    fname=peak.assignment+'.dat'
    fid=open(os.path.join(path,fname), 'w')
    fid.write(output)
    fid.close()
    #- X-vector
    fname=peak.assignment+'.x'
    fid=open(os.path.join(path,fname), 'w')
    fid.write(X_line)
    fid.close()
    #- Y-vector
    fname=peak.assignment+'.y'
    fid=open(os.path.join(path,fname), 'w')
    fid.write(Y_line)
    fid.close()
    
    
    
    
    
    
    #--- find all peaks in this area for labeling
    all_peaks=spectrum.peak_list()
    neighbours=""
    for extra_peak in all_peaks:
        if extra_peak.position[Xdim] < (peak.position[Xdim]+ dw[Xdim]) and extra_peak.position[Xdim] > (peak.position[Xdim]- dw[Xdim]) :
            if extra_peak.position[Ydim] < (peak.position[Ydim]+ dw[Ydim]) and extra_peak.position[Ydim] > (peak.position[Ydim]- dw[Ydim]):
            
                #-- compute index of the peak in the full spectral range
                ind_peak=[0,0]
                ind_peak[Xdim]= ( extra_peak.position[Xdim] -  omega_min[Xdim])/ dw_grid[Xdim] + 1
                ind_peak[Ydim]= ( extra_peak.position[Ydim] -  omega_min[Ydim])/ dw_grid[Ydim] + 1
                            
               
                #-- collect peak
                neighbours+="%s\t%f\t%f\n" % (extra_peak.assignment, ind_peak[Xdim], ind_peak[Ydim])
    
    #-- write list of neighbours
    fid=open(os.path.join(path,peak.assignment+'.neighbours'),'w')
    fid.write(neighbours)
    fid.close()
   
    print "'%s' exported to %s" % (peak.assignment,path)
    
    return 




