doc=""" -----------------------------------------------------------------------------
Grab Peaks.

The gadget to record peak width and print it so Evgenii can cut-and-paste into
his Gnumeric tables 

w1,w2,w3 - defines working axis

Dragging a box defines a ppm range to grab along the current "w" axis and displays it.

High,Medium, Low - arbitrary intensity measure:  adds H, M or L modifier to it

"""

import sputil,string
import tkutil, tkMessageBox

# -----------------------------------------------------------------------------
#
axis={0:'w1',1:'w2',2:'w3'}

class region_rmsd_dialog(tkutil.Dialog):
  # storage for mean RMSD value
  _current_spectrum=None
  _current_axis=0 #-- default w1
  _intensity='H'
  def __init__(self, session):

    self.session = session
    self.drag_notice = None
    
    tkutil.Dialog.__init__(self, session.tk, 'Grab Peaks v.1')

    self.top.columnconfigure(0, weight = 1)

    import Tkinter
    explain = ('Define the current axis and drag a box around the peak.\n\
See Python Shell window for cut-and-paste result\n\
------------------------------------------------')
    w = Tkinter.Label(self.top, text = explain, justify = 'left')
    w.grid(row = 0, column = 0, sticky = 'nw')

    r1 = Tkinter.Label(self.top, justify = 'left')
    r1.grid(row =1 , column = 0, sticky = 'nw')
    self.axis_label = r1

    r2 = Tkinter.Label(self.top, justify = 'left')
    r2.grid(row =2 , column = 0, sticky = 'nw')
    self.intensity_label = r2

    self.axis_label['text'] = "Current axis - %s" % axis[self._current_axis]
    self.intensity_label['text'] = "Current intensity - High"

    r = Tkinter.Label(self.top, justify = 'left')
    r.grid(row = 3, column = 0, sticky = 'nw')
    self.result_label = r

    br = tkutil.button_row(self.top,
                           ('w1',self.set_w1),
                           ('w2',self.set_w2),
                           ('w3',self.set_w3),
			   ('High',self.high_intensity),
                           ('Medium',self.medium_intensity),
                           ('Low',self.low_intensity),
                           ('Close', self.close_cb),
                           ('Help', self.help)
			   )
    br.frame.grid(row = 4, column = 0, sticky = 'nw')

    self.register_callback(1)
    self.top.bind('<Map>', lambda e, s=self: s.register_callback(1))
    self.top.bind('<Unmap>', lambda e, s=self: s.register_callback(0))

  #--------------------------------------------------------------
  def help(self):
    self.session.command_characters('py')
    print "\n\n==========================================="
    print doc
    print "============= see help message above ! =======\n"

  #----------------------------
  def high_intensity(self):
    self._intensity='H'
    self.intensity_label['text'] = "Current intensity - High"

  def medium_intensity(self):
    self._intensity='M'
    self.intensity_label['text'] = "Current intensity - Medium"

  def low_intensity(self):
    self._intensity='L'
    self.intensity_label['text'] = "Current intensity - Low" 




  # ---------------------------------------------------------------------------

  #
  def set_w1(self):
    self._current_axis=0
    self.axis_label['text'] = "Current axis - %s" % axis[self._current_axis]
    
  def set_w2(self):
    self._current_axis=1
    self.axis_label['text'] = "Current axis - %s" % axis[self._current_axis]
    
  def set_w3(self):
    self._current_axis=2
    self.axis_label['text'] = "Current axis - %s" % axis[self._current_axis]

  #--------------------------------------------------------
  def register_callback(self, register):

    if register:
      if self.drag_notice == None:
        self.drag_notice = self.session.notify_me('drag region',
                                                  self.region_selected_cb)
    else:
      if self.drag_notice:
        self.session.dont_notify_me(self.drag_notice)
        self.drag_notice = None

  # ---------------------------------------------------------------------------
  #
  def region_selected_cb(self, region):

    if self.is_window_destroyed():
      self.register_callback(0)
      return

    s = self.session.selected_spectrum()
    if s == None:
      self.result_label['text'] = 'No spectrum selected'
      return
    rmin, rmax = region
    ranges = map(lambda vmin, vmax: '%.4g-%.4g' % (vmin, vmax), rmin, rmax)
    rtext = '[ ' + string.join(ranges, ', ') + ' ]'

    if self._current_axis>len(ranges)-1:
      tkMessageBox.showerror("Grab Peak","The spectrum does not have '%s' axis!"%axis[self._current_axis])
      return

    grabbed_region=ranges[self._current_axis]

  
    self.session.command_characters('py')
 
    minppm,maxppm=map(float,grabbed_region.split('-'))
    print '%.2f-%.2f:%s' %(maxppm,minppm,self._intensity)


#------------------------------------------------------------------------------------
def run(session):

  sputil.the_dialog(region_rmsd_dialog,session).show_window(1)
