#!/usr/bin/env python
#-----  Sparky extension for runnining type_prob program for prediction of amino-acid residue type
# Evgenii Kovrigin, Medical College of Wisconsin
# v.1    11/3/08


doc="""Prediction of the amino acid type on a basis of C-alpha and C-beta chemical shifts using "type_prob" described in "Amino Acid Type determination in the sequential assignment procedure of 
uniformly 13C/15N-enriched proteins" S. Grzesiek, A. Bax, J. Biomol. NMR 1993, 3, 185-204.

How to use the extension:
(1) Select C-alpha peak and type 'et'.
(2) Select C-beta and type 'et' again. See prediction of the residue type in a Python shell. 
(3) Type 'et' again to reset the counter and start over."""


#-- path to the program type_prob 
type_prob='type_prob'


#--- this is my global storage of selected peaks and frequencies
class PeakSet:
    peaks_number=0
    Ca_shift=0
    Cb_shift=0
    
    

import tkMessageBox, tkFileDialog, tkSimpleDialog
import os, sys
tkMessageBox.showinfo('Predict Amino Acid Type',doc)
print doc 

def run(session):
    session.command_characters('py')
        
    
    
    
    curspec=session.selected_spectrum()
    selected=curspec.selected_peaks()
    
    if len(selected)!=1:
      tkMessageBox.showerror('Predict Amino Acid Type',"Need one carbon peak selected!")
      return
    

    #-- find which axis is carbon
    nuclei=session.selected_spectrum().nuclei
    carbon_axis=0
    for nucleus in nuclei:
        if nucleus=='13C':
            break
        else:
            carbon_axis+=1
    
    Cppm=selected[0].position[carbon_axis]
    
    
    if PeakSet.peaks_number==0:
        PeakSet.Ca_shift=Cppm
        PeakSet.peaks_number+=1
        
    else:
        if PeakSet.peaks_number==1:
            PeakSet.Cb_shift=Cppm
            PeakSet.peaks_number+=1
            
            
        else:
            if PeakSet.peaks_number==2:
                print 'Reset peak picking.\n\n'
                PeakSet.peaks_number=0
                PeakSet.Ca_shift=0
                PeakSet.Cb_shift=0
              
  
    print '\n\nPrediction of amino acid type using "type_prob"'
    print 'Peaks captured: %d' % PeakSet.peaks_number
    print 'Calpha shift: %.2f' % PeakSet.Ca_shift
    print 'Cbeta shift: %.2f' % PeakSet.Cb_shift
    
    if PeakSet.peaks_number==2:
        #-- time to run type_prob
        output=os.popen('%s %f %f' % (type_prob, PeakSet.Ca_shift, PeakSet.Cb_shift) ,'r')
        print 'Probability of different residues'
        lines=output.readlines()
        for line in lines:
            print line.strip()
     