import tkSimpleDialog
import tkMessageBox

doc="Time/Index value recalculation.\n\
written by Evgenii Kovrigin   04/01/05\n\
---------------\n\
Usage:\n\
(1) Have your time series spectra loaded.\n\
(2) Open 'rh','es' or 'eh' Setup window.\n\
(3) Insert number of cycles for each spectrum.\n\
(4) Hit 'Apply'.\n\
Then: hit 'OK' in this window and enter experiment type and tau value.\n Calculated \
times are displayed in the Python Shell window.\n Cut-and-Paste them into \
the Setup window of 'eh','es' or 'rh'.\n Hit 'Apply' - proceed with data fitting.\n\n\
In order to add new equations for other experiments:\n (1) open 'recalculate_times.py'\n\
(2) add experiment name and equation to a dictionary 'equations'.\
"



index_name='relaxation_time_parameter'

#--- ADD YOUR NEW EQUATION HERE: if times=tau*cycles use "new":"tau*cycles"
#   If you need additional variables, edit the code below to allow entering them
equations={"R1":"cycles*tau+2",
           "R2":"4*tau*(cycles+1)"}

def run(session):
    """ converts 'saved value' by some formula"""
    tkMessageBox.showinfo("Index value conversion",doc)

    spectra=session.project.spectrum_list()
    if not spectra:
        tkMessageBox.showwarning("Index value conversion","\n\n!!! Error!  Need some spectra loaded !!!\n\n")
        return
    print "\n","-"*20
#=====================================================================

    string="Conversion of index value to a time, seconds.\n"
    string+="Available equations:\nname : equation\n"
    
    #--- Show available equations ---
    for name in equations.keys():
        string+= name+ " : time="+ equations[name]+ "\n"
    string+="\n"

    #--- Ask for the equation name ---
    name=tkSimpleDialog.askstring("Index value conversion",string+"Enter name of experiment:")
    if not name: return
    if name not in equations.keys():
        tkMessageBox.showwarning("Index value conversion","Formula for '%s' was not found! Exiting...\n" % name)
        return

#--- HERE YOU REQUEST ADDITIONAL VARIABLES FROM USER ---
    #--- Choose tau ----
    if name in ("R1","R2"):
        tau=float(tkSimpleDialog.askfloat("Index value conversion",
            "Experiment: %s\nEquation is: time=%s\nEnter 'tau' value in ms:" % (name,equations[name])))

    #--- Calculate ----
    string=""
    for s in spectra:
        index=s.saved_value(index_name)
        cycles=float(index)
        value=eval(equations[name])*1e-3
        string+='"%s": %s => %.4f \n' % (s.name,index,value)
        
    session.command_characters('py')
    print "Copy-and-paste into eh,es or rh window!"
    print "Experiment: %s\nEquation: time=%s\nSpectrum name\t:\tindex\t=>\tnew_value\n%s" %(name,equations[name],string)
            
        



