""" Sparky extension 'Update assignments'
v.1 by Evgenii Kovrigin         11/21/02

The concept is:
  - you have an assignment file ('master') which is considered to be a storage
    place for your most current assignments
    
  - you opened a few Sparky instances for work with different kind of spectra
    on different desktops.

  - Based on results you change assignment positions in the 'master' window

  - you want them to be reflected in other Sparky sessions 'immediately' so that you
    can begin using your latest result without manual adjustments or reloading of
    the files.

    Here is one of all possible ways to do so:
    
      -you change your 'master' window and save it
      
      -select 'slave' window (in a different Sparky session)
      
      -press 'eu' - update assignment POSITIONS.
      
      NOTE: 1. if you DELETED  some peaks in the 'master' - I cannot automatically
               delete them in your 'slave' window but instead you get them all
               selected and you can delete them with just one click of [Del] key

            2. if you ADDED new assignments in the 'master' session - I cannot ADD
               new peak assignments but only add a peak and open a list of them for you:
               click through these new peaks and do 'at' to add the name manually

  - Of course, this all is not intended to make MASSIVE ADDITIONS of the assignments -
    in that case it is easier to use another utility - command line 'make_sp_files.py'
"""

import textfile
import tkFileDialog
import tkMessageBox

def run(session):
    """ read assignments """

    session.command_characters("py")
    print "\n\n\n*************** Updating assignments  *********************"
    spectra=session.project.spectrum_list()
    #--- check if window is open ----
    if not spectra:

        print "\n\n!!! Error!  Need some spectrum loaded !!!\n\n"
        return

    #-- get fresh assignments ----
    spec=session.selected_view().spectrum
   

    #-- get the name of file with right assignments ---
    current_fname=spec.saved_value("assignment_fname")
    
    #---- no previous name ? -----
    if not current_fname:
        print "Choose the file with the most recent assignments, please"
        assign_fname=tkFileDialog.askopenfilename()
        spec.save_value("assignment_fname",assign_fname)

    #--- like previous assignments ? ----
    elif tkMessageBox.askyesno("Question","Updated assignments are to be read from %s?" % current_fname):
        assign_fname=current_fname

    #--- load new name anyway ---
    else:
        assign_fname=tkFileDialog.askopenfilename()
        spec.save_value("assignment_fname",assign_fname)
        
    if not assign_fname: return  # [Cancel] pressed
    
    print "Read the most current assignments from:\n\
    '%s'\n*****************************************" % assign_fname
   
    new_assignments=textfile.Sparky_Data_File(assign_fname,0).get_positions()

    #--- update assignments in the current window ---
    to_delete=[]
    resnames_to_delete=""
    for peak in spec.peak_list():
        peak.selected=0
        resname=peak.assignment[:-3]
        if resname not in new_assignments.get_keys():
            to_delete.append(peak)
            resnames_to_delete+="  " + resname
            continue
        peak.position=new_assignments[resname]
        
        #-- remove one dealt with ---
        del new_assignments[resname]
        
    
    #--- place new peaks if any ---
    to_add=""
    if new_assignments.sorted_keys():
        new_peaks=[]
        for res in new_assignments.sorted_keys():
            N,H=new_assignments[res]
            new_peaks.append(spec.place_peak([N,H]))
            to_add+="%s\t%.4f\t\t%.4f\n" % (res,N,H)
        session.show_peak_list(new_peaks)       

        print "\nNew assigned peaks are listed\n (please, do 'at' manually for them!): \n---------------------------------------------\n%s" % to_add 
    else:
        print "No assignments to add..."    


    #-- center them all and unselect ----
    session.command_characters("pa")
    #session.command_characters("pc")
    for peak in session.selected_peaks():  peak.selected=0

        
    #--- show peaks-to-delete if any --
    if to_delete:
        #-- show peaks to delete ---
        print "\nPeaks that are not present in the most current assignments are selected:\n\
(please, delete them manually)\n--------------------------\n%s" % resnames_to_delete
        session.show_peak_list(to_delete)
        for peak in to_delete:  peak.selected=1
    else:
        print "No assignments to delete..."    


